## Introduction

This is a helper tool for fetch content from Flow Framework / Neos CMS installation on a remote server to your local
computer running with docker.

## Dependency

This is requires that your Flow Framework / Neos CMS installation is configured to run with `dockerflow` package
(see https://github.com/Sebobo/Shel.DockerFlow) and all containers are started before executing the sync.

## Installation

Add the `syncontent` package into your `composer.json` file and update `composer.lock`. It is recommended to add in the
`require-dev` section of your composer.

```
"visay/syncontent": "dev-master"
```

## Usage

In the root directory of your project, execute:

```bash
bin/syncontent --remote-user=demo-014-007
```

- Replace the user `demo-014-007` with the one you want to get content from
- You need to make sure that you have auto login with public key to `demo-014-007@YOUR_SERVER_HOSTNAME_OR_IP`.
The default host server is `10.10.10.27`, which is the internal web server
- You can define different server host with `--remote-host=YOUR_SERVER_HOSTNAME_OR_IP` argument
- Only `latest` or `demo` is supported at the moment. We don't recommend you to have autologin access to the live site
- You can only run the command from the root directory of your project
- The document root on the server will have to be at `/home/demo-014-007/public_html`
- If you have a different document root path, you can overwrite the default with `--remote-path` argument

So the full overwriting way would be:

```bash
bin/syncontent --remote-user=demo-014-007 --remote-host=10.10.10.37 --remote-path=/home/user/neosbox
```

Or short form of the arguments:

```bash
bin/syncontent -u=demo-014-007 -h=10.10.10.37 -p=/home/user/neosbox
```

### Available arguments

- `-u` | `--remote-user` : Set the login user to the remote server (__Required__)
- `-h` | `--remote-host` : Set the hostname or IP address of the remote server (__Default__: `10.10.10.27`)
- `-p` | `--remote-path` : Set path to the document root on the remote server (__Default__: `/home/<remote-user>/public_html`)
- `-t` | `--ansible-tags`: Set the specific task you want run. See available sysc tasks section for details
- `--remote-php`         : Set the path to php on the remote server
- `-v`                   : Set the verbosity of ansible output. You can also set more verbose by adding more `v` like `-vvvvv`
- `--dry-run`            : Force to display the command to execute but won't run them. Useful for debugging when you want
                           to see the full ansible command and maybe execute them directly for better error hints

### Customization

If you are lazy typing the user again and again, you can define it permanently by creating a file in
`Packages/Libraries/visay/syncontent/config/master` with the same name as the `--remote-user` value.

```
Packages
└── Libraries
	└── visay
		└── syncontent
			├── ansible
			├── bin
			├── composer.json
			├── config
			│   └── master
			│       └── demo-014-007
			└── README.md
```

You can also overwrite the default value of `--remote-host`, `--remote-path` and `--remote-php` by setting it in the created master file.

```Packages/Libraries/visay/syncontent/config/master/demo-014-007
--remote-host=10.10.10.37
--remote-path=/home/user/neosbox
--remote-php=/opt/php-versions/php55/bin/php
```

With this file and its content, you can now execute the content sync with just:

```bash
bin/syncontent
```

And the script will automatically take `demo-014-007` as the content master, `10.10.10.37` as server host and
`/home/user/neosbox` as remote path.

- You can also combine that some settings are in config file and some are passed as argument
- If the same config are set in the config and pass as argument, the argument value will take effect
- There should be only one master at a time. That means you should not have more than more file in the `config/master`
directory. In case there are more than one file exists, the script will take first file only to process.
- In most case, the default host and path work fine for internal Web Essentials project development
and you just need to define the `--remote-user`

### Available Sync Tasks

Syncontent does its job in 3 different batches. First it sync the user files from remote server to local. Second it dumps
the database from remote and restore on local and last it runs some necessary flow commands for clean up.

By default, syncontent will run all 3 tasks but you can pass the parameter to call specific task only that you want to run.

```bash
bin/syncontent --ansible-tags=<TASK-NAME>
```

The available task names are:

- rsync
- db
- flow

### 1. rsync

The first task is to sync directory content in `Data/Persistent/` from remote to local. Files that exists only in local
will be removed to make sure everything is exactly the same as on remote.

To execute this task only, run the command:

```bash
bin/syncontent --ansible-tags=rsync
```

### 2. db

The second task is to reset the local database. It will start by connecting to remote server and get the database connection
information for the Production context of Flow. You cannot change to other context at the moment so if your remote is not
running in Production context, this tool won't work for you but anyway tweaking it to work is not that hard
and you can do it in the ansible role.

Once it gets all the information, it will dump the database, zip it and download to local temp. Before restoring to local
database, it first deletes the local database completely and recreate it again. This way assures that we have exact
state as on remote and prevent from any issues with the migration status.

To execute this task only, run this command:

```bash
bin/syncontent --ansible-tags=db
```

### 3. flow

The last step is to run some necessary commands needed in Flow for the application to work. They run in the following order:

- flow:cache:flush --force
- database:setcharset
- doctrine:migrate
- resource:publish
- node:repair
- cache:warmup

To execute this task only, run this command:

```bash
bin/syncontent --ansible-tags=flow
```

## Author

Visay Keo <visay@web-essentials.asia>

If you have any feedback, comments and/or questions, feel free to contact with email address above. And of course,
a merge request is always welcomed.
